package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.entity.TBSyncTask;
import com.qmth.distributed.print.business.mapper.TBSyncTaskMapper;
import com.qmth.distributed.print.business.service.TBSyncTaskService;
import com.qmth.teachcloud.common.bean.result.SyncListResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PushTypeEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

/**
 * @Date: 2021/5/20.
 */
@Service
public class TBSyncTaskServiceImpl extends ServiceImpl<TBSyncTaskMapper, TBSyncTask> implements TBSyncTaskService {
    @Resource
    TBSyncTaskMapper tbSyncTaskMapper;

    @Override
    public TBSyncTask saveTask(ExamPrintPlan examPrintPlan) {
        QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBSyncTask::getSchoolId, examPrintPlan.getSchoolId())
                .eq(TBSyncTask::getPrintPlanId, examPrintPlan.getId())
                .eq(TBSyncTask::getType, PushTypeEnum.EXAM_PUSH)
                .ne(TBSyncTask::getStatus, TaskStatusEnum.FINISH);
        TBSyncTask tbSyncTask = this.getOne(queryWrapper);
        if (tbSyncTask != null) {
            throw ExceptionResultEnum.ERROR.exception("计划下有数据正在同步");
        } else {
            tbSyncTask = new TBSyncTask();
            tbSyncTask.setId(SystemConstant.getDbUuid());
            tbSyncTask.setSchoolId(examPrintPlan.getSchoolId());
            tbSyncTask.setPrintPlanId(examPrintPlan.getId());
            tbSyncTask.setPrintPlanName(examPrintPlan.getName());
            tbSyncTask.setType(PushTypeEnum.EXAM_PUSH);
            tbSyncTask.setStatus(TaskStatusEnum.INIT);
            tbSyncTask.setCreateTime(System.currentTimeMillis());
        }
        this.save(tbSyncTask);
        return tbSyncTask;
    }

    @Transactional
    @Override
    public TBSyncTask savePushCommon(PushTypeEnum pushTypeEnum, Map map, SysUser sysUser) {
        SysUser requestUser = Objects.nonNull(sysUser) ? sysUser : (SysUser) ServletUtil.getRequestUser();
        TBSyncTask tbSyncTask = new TBSyncTask(pushTypeEnum,
                TaskStatusEnum.INIT,
                requestUser.getId(),
                requestUser.getSchoolId());

        map.computeIfAbsent(SystemConstant.TB_SYNC_TASK, v -> tbSyncTask);
        map.computeIfAbsent(SystemConstant.USER, v -> requestUser);
        map.computeIfAbsent(SystemConstant.TB_SYNC_TASK_ID, v -> tbSyncTask.getId());
        return tbSyncTask;
    }

    @Override
    public IPage<SyncListResult> query(Long schoolId, TaskStatusEnum status, PushTypeEnum type, TaskResultEnum result, int pageNumber, int pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Set<Long> orgIds = null;
        IPage<SyncListResult> page = tbSyncTaskMapper.query(new Page<>(pageNumber, pageSize), schoolId, status, type, result, sysUser.getId(), orgIds);
        List<SyncListResult> list = page.getRecords();
        for (SyncListResult syncListResult : list) {
            syncListResult.setStatus(Objects.nonNull(syncListResult.getStatus()) ? TaskStatusEnum.valueOf(syncListResult.getStatus()).getTitle() : null);
            syncListResult.setType(Objects.nonNull(syncListResult.getType()) ? PushTypeEnum.valueOf(syncListResult.getType()).getTitle() : null);
            syncListResult.setResult(Objects.nonNull(syncListResult.getResult()) ? TaskResultEnum.valueOf(syncListResult.getResult()).getTitle() : null);
        }
        return page;
    }
}
