package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.entity.TExamTaskFlow;
import com.qmth.distributed.print.business.mapper.TExamTaskFlowMapper;
import com.qmth.distributed.print.business.service.TExamTaskFlowService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * 考试任务和流程关联表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-11-22
 */
@Service
public class TExamTaskFlowServiceImpl extends ServiceImpl<TExamTaskFlowMapper, TExamTaskFlow> implements TExamTaskFlowService {

    @Resource
    RedisUtil redisUtil;

    @Resource
    TExamTaskFlowMapper taskFlowMapper;

    @Resource
    CommonCacheService commonCacheService;

    /**
     * 保存TExamTaskFlow
     *
     * @param map
     */
    @Override
    public boolean saveOrUpdate(Map<String, Object> map) {
        Long schoolId = (Long) map.get(SystemConstant.SCHOOL_ID);
        Long taskId = (Long) map.get(SystemConstant.EXAM_TASK_ID);
        Long flowId = (Long) map.get(SystemConstant.FLOW_ID);
        boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_EXAM_TASK_FLOW_PREFIX + flowId, SystemConstant.REDIS_LOCK_EXAM_TASK_FLOW_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("该流程正在审批中，请稍候再试！");
        }
        try {
            QueryWrapper<TExamTaskFlow> taskFlowQueryWrapper = new QueryWrapper<>();
            taskFlowQueryWrapper.lambda().eq(TExamTaskFlow::getTaskId, taskId);
            List<TExamTaskFlow> tExamTaskFlowList = this.list(taskFlowQueryWrapper);
            if (Objects.isNull(tExamTaskFlowList) || tExamTaskFlowList.size() == 0) {
                TExamTaskFlow tExamTaskFlow = new TExamTaskFlow(taskId, flowId, 1);
                this.save(tExamTaskFlow);
            } else if (Objects.nonNull(tExamTaskFlowList) && tExamTaskFlowList.size() == 1) {
                Map<String, List<String>> flowSchoolCodeMap = SystemConstant.getFlowSchoolCodeMap();
                BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
                if (flowSchoolCodeMap.get(SystemConstant.GDYKDX_FLOW_KEY).contains(basicSchool.getCode())) {
                    TExamTaskFlow tExamTaskFlow = tExamTaskFlowList.get(0);
                    Integer level = tExamTaskFlow.getLevel();
                    TExamTaskFlow tExamTaskFlowSub = new TExamTaskFlow(taskId, flowId, ++level);
                    this.save(tExamTaskFlowSub);
                }
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(SystemConstant.REDIS_LOCK_EXAM_TASK_FLOW_PREFIX + flowId);
        }
        return true;
    }

    /**
     * 根据考试任务id查找
     *
     * @param taskId
     * @return
     */
    @Override
    public TExamTaskFlow findByFlowLevel(Long taskId) {
        Integer level = taskFlowMapper.findByFlowLevel(taskId);
        Optional.ofNullable(level).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到子流程信息"));
        QueryWrapper<TExamTaskFlow> tExamTaskFlowQueryWrapper = new QueryWrapper<>();
        tExamTaskFlowQueryWrapper.lambda().eq(TExamTaskFlow::getTaskId, taskId)
                .eq(TExamTaskFlow::getLevel, level);
        return this.getOne(tExamTaskFlowQueryWrapper);
    }

    /**
     * 根据考试任务id查找
     *
     * @param taskId
     * @return
     */
    @Override
    public List<TExamTaskFlow> findByExamTaskId(Long taskId) {
        return this.list(new QueryWrapper<TExamTaskFlow>().lambda().eq(TExamTaskFlow::getTaskId, taskId));
    }

    /**
     * 根据流程id查找
     *
     * @param flowId
     * @return
     */
    @Override
    public TExamTaskFlow findByRootLevel(Long flowId) {
        Long examTaskId = taskFlowMapper.findByRootLevel(flowId);
        Optional.ofNullable(examTaskId).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到考试任务id信息"));
        Integer level = taskFlowMapper.findByRootLevelByExamTaskId(examTaskId);
        Optional.ofNullable(level).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到主流程信息"));
        QueryWrapper<TExamTaskFlow> tExamTaskFlowQueryWrapper = new QueryWrapper<>();
        tExamTaskFlowQueryWrapper.lambda().eq(TExamTaskFlow::getTaskId, examTaskId)
                .eq(TExamTaskFlow::getLevel, level);
        return this.getOne(tExamTaskFlowQueryWrapper);
    }
}
