package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.result.FlowApproveResult;
import com.qmth.distributed.print.business.entity.TFFlow;
import com.qmth.distributed.print.business.entity.TFFlowLog;
import com.qmth.distributed.print.business.mapper.TExamTaskFlowMapper;
import com.qmth.distributed.print.business.mapper.TFFlowLogMapper;
import com.qmth.distributed.print.business.mapper.TFFlowMapper;
import com.qmth.distributed.print.business.service.TFFlowService;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.FlowApproveSetupEnum;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 流程表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-08-02
 */
@Service
public class TFFlowServiceImpl extends ServiceImpl<TFFlowMapper, TFFlow> implements TFFlowService {

    @Resource
    TFFlowMapper tfFlowMapper;

    @Resource
    TExamTaskFlowMapper tExamTaskFlowMapper;

    @Resource
    TFFlowLogMapper tfFlowLogMapper;

    @Autowired
    SysUserService sysUserService;

    /**
     * 流程列表
     *
     * @param iPage
     * @param name
     * @param schoolId
     * @param orgId
     * @return
     */
    @Override
    public IPage<TFFlow> list(IPage<Map> iPage, String name, Long schoolId, Long orgId) {
        return tfFlowMapper.list(iPage, name, schoolId, orgId);
    }

    /**
     * 流程审批记录列表
     *
     * @param iPage
     * @param startName
     * @param schoolId
     * @param orgId
     * @param flowId
     * @return
     */
    @Override
    public IPage<FlowApproveResult> flowApproveList(IPage<Map> iPage, String status, String startName, Long teachingRoomId, Long createStartTime, Long createEndTime, Long schoolId, Long orgId, Long flowId) {
        IPage<FlowApproveResult> flowApproveResultIPage = tfFlowMapper.flowApproveList(iPage, status, startName, teachingRoomId, createStartTime, createEndTime, schoolId, orgId, flowId);
        for (FlowApproveResult record : flowApproveResultIPage.getRecords()) {
            record.setStatusStr(record.getStatus().getTitle());
            // FlowApproveSetupEnum
            if (FlowApproveSetupEnum.FINISH.getSetup() == record.getSetup()) {
                record.setSetupStr("结束节点");
            } else if (FlowApproveSetupEnum.SUBMIT.getSetup() == record.getSetup()) {
                record.setSetupStr("开始节点");
            } else {
                record.setSetupStr("审批节点");
            }

            Long maxFlowId = tExamTaskFlowMapper.selectMaxFlowIdByExamTaskId(record.getTaskId());
            if (Objects.nonNull(maxFlowId)) {
                List<TFFlowLog> flowLogs = tfFlowLogMapper.findByFlowId(maxFlowId, Long.parseLong((String) ServletUtil.getRequestHeaderSchoolId()));
                TFFlowLog tfFlowLog = flowLogs.stream().sorted(Comparator.comparingLong(TFFlowLog::getCreateTime).reversed()).findFirst().orElse(null);
                if (Objects.nonNull(tfFlowLog)) {
                    String pendApproveId = tfFlowLog.getPendApproveId();
                    if (StringUtils.isNotBlank(pendApproveId)) {
                        StringJoiner sj = new StringJoiner(",");
                        for (String id : pendApproveId.split(",")) {
                            SysUser sysUser = sysUserService.getById(id);
                            sj.add(sysUser.getRealName());
                        }
                        record.setCurrentApproveUserName(sj.toString());
                    }
                }
            }

        }
        return flowApproveResultIPage;
    }

    /**
     * 流程审批记录找人
     *
     * @param schoolId
     * @param flowId
     * @return
     */
    @Override
    public FlowApproveResult flowApproveUserList(Long schoolId, Long flowId) {
        return tfFlowMapper.flowApproveUserList(schoolId, flowId);
    }

    /**
     * 根据key查找流程
     *
     * @param flowKey
     * @return
     */
    @Override
    public Integer findByFlowKey(String flowKey) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TFFlow> tfFlowQueryWrapper = new QueryWrapper<>();
        tfFlowQueryWrapper.lambda().eq(TFFlow::getSchoolId, sysUser.getSchoolId())
                .eq(TFFlow::getFlowKey, flowKey)
                .eq(TFFlow::getEnable, true)
                .eq(TFFlow::getPublish, true);
        return this.count(tfFlowQueryWrapper);
    }

    /**
     * 根据key查找id
     *
     * @param flowKey
     * @return
     */
    @Override
    public String findActIdByFlowKey(String flowKey) {
        return tfFlowMapper.findActIdByFlowKey(flowKey);
    }
}
