package com.qmth.distributed.print.business.templete.export;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.templete.importData.AsyncImportTaskTemplete;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.TBTaskService;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 异步导出模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
public abstract class AsyncExportTaskTemplete {
    private final static Logger log = LoggerFactory.getLogger(AsyncImportTaskTemplete.class);
    public static final String BEGIN_TITLE = "->开始准备处理导出的";
    public static final String FINISH_TITLE = "->数据处理结束，共处理了";
    public static final String FINISH_SIZE = "条数据。";
    public static final String EXCEPTION_TITLE = "->数据处理发生异常!";
    public static final String EXCEPTION_DATA = "错误信息:";
    public static final String TXT_PREFIX = ".txt";
    public static final String EXCEPTION_CREATE_TXT_TITLE = "->创建导出日志时发生异常!";

    /**
     * 异步导出任务
     *
     * @param map
     * @return
     */
    @Async
    public abstract Result exportTask(Map<String, Object> map) throws Exception;

    /**
     * 创建txt文件
     *
     * @param tbTask
     */
    public void createTxt(TBTask tbTask) {
//        OssUtil ossUtil = SpringContextHolder.getBean(OssUtil.class);
        FileStoreUtil fileStoreUtil = SpringContextHolder.getBean(FileStoreUtil.class);
        TBTaskService tbTaskService = SpringContextHolder.getBean(TBTaskService.class);
        ByteArrayOutputStream out = null;
        InputStream inputStream = null;
        try {
            JSONObject jsonObject = JSONObject.parseObject(tbTask.getResultFilePath());
            if (Objects.isNull(jsonObject)) {
                jsonObject = new JSONObject();
                LocalDateTime nowTime = LocalDateTime.now();
                StringJoiner stringJoiner = new StringJoiner("");
                stringJoiner.add(UploadFileEnum.FILE.getTitle()).add(File.separator);
                stringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                        .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                        .add(String.format("%02d", nowTime.getDayOfMonth()))
                        .add(File.separator).add(SystemConstant.getUuid()).add(TXT_PREFIX);
                jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
                jsonObject.put(SystemConstant.PATH, stringJoiner.toString());
            }
            String path = (String) jsonObject.get(SystemConstant.PATH);
            path = path.replaceAll("\\\\","/");
            String type = (String) jsonObject.get(SystemConstant.TYPE);

            out = new ByteArrayOutputStream();
            out.write(tbTask.getSummary().getBytes(StandardCharsets.UTF_8));
            byte[] bookByteAry = out.toByteArray();
            inputStream = new ByteArrayInputStream(bookByteAry);
            StringJoiner stringJoiner = new StringJoiner("");
            if (Objects.equals(type, SystemConstant.OSS)) {//上传至oss
                path = path.substring(0, path.lastIndexOf("/") + 1);
                stringJoiner.add(path).add(SystemConstant.getUuid()).add(TXT_PREFIX).toString();
//                ossUtil.ossUpload(stringJoiner.toString(), inputStream, null);
                fileStoreUtil.ossUpload(stringJoiner.toString(), inputStream, DigestUtils.md5Hex(new ByteArrayInputStream(bookByteAry)), fileStoreUtil.getUploadEnumByPath(stringJoiner.toString()).getFssType());
            } else {//上传至服务器
                path = path.substring(0, path.lastIndexOf(File.separator) + 1);
                stringJoiner.add(path).add(SystemConstant.getUuid()).add(TXT_PREFIX).toString();
                File finalFile = new File(stringJoiner.toString());
                if (!finalFile.exists()) {
                    finalFile.getParentFile().mkdirs();
                    finalFile.createNewFile();
                }
                FileUtils.copyInputStreamToFile(inputStream, finalFile);
            }
            JSONObject json = new JSONObject();
            json.put(SystemConstant.PATH, stringJoiner.toString());
            json.put(SystemConstant.TYPE, type);
            json.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
            String result = json.toJSONString();
            tbTask.setReportFilePath(json.toJSONString());
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            StringJoiner stringJoinerSummary = new StringJoiner("").add(tbTask.getSummary()).add("\n");
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), EXCEPTION_CREATE_TXT_TITLE, EXCEPTION_DATA, e.getMessage()));
            tbTask.setSummary(stringJoinerSummary.toString());
            tbTask.setResult(TaskResultEnum.ERROR);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            try {
                if (Objects.nonNull(inputStream)) {
                    inputStream.close();
                }
                if (Objects.nonNull(out)) {
                    out.flush();
                    out.close();
                }
            } catch (IOException e) {
                log.error(SystemConstant.LOG_ERROR, e);
            }
            tbTask.setStatus(TaskStatusEnum.FINISH);
            tbTaskService.updateById(tbTask);
        }
    }
}
